{*******************************************************}
{                                                       }
{              Format SVG XML v1.1                      }
{                                                       }
{              Copyright(c) 2023 PA-Soft e.U.           }
{              https://www.pa-soft.com                  }
{                                                       }
{              All rights reserved                      }
{                                                       }
{*******************************************************}

unit uFormatSVGXML;

interface

function FormatSVGXML(const AXML: string): string;

implementation

uses
  System.SysUtils,
  Xml.XMLDoc, Xml.XMLIntf;

function FormatNode(ANode: Xml.XMLIntf.IXMLNode; Indent: string = ''): string;
var
  I: Integer;
  ChildNode: IXMLNode;
  AttrNode: IXMLNode;
  AttrText: string;
begin
  Result := '';

  if ANode.NodeType = ntText then
  begin
    Result := Trim(ANode.Text);
    Exit;
  end;

  if ANode.NodeType = ntElement then
  begin
    AttrText := '';

    for I := 0 to ANode.AttributeNodes.Count - 1 do
    begin
      AttrNode := ANode.AttributeNodes.Nodes[I];
      AttrText := AttrText + Format(' %s="%s"', [AttrNode.NodeName, AttrNode.NodeValue]);
    end;

    Result := Indent + '<' + ANode.NodeName + AttrText;

    if ANode.HasChildNodes then
    begin
      Result := Result + '>' + sLineBreak;

      for I := 0 to ANode.ChildNodes.Count - 1 do
      begin
        ChildNode := ANode.ChildNodes.Nodes[I];
        Result := Result + FormatNode(ChildNode, Indent + '  ') + sLineBreak;
      end;

      Result := Result + Indent + '</' + ANode.NodeName + '>';
    end
    else
    begin
      Result := Result + ' />';
    end;
  end;
end;

function FormatSVG(const AXML: string): string;
var
  LXMLDocument: Xml.XMLIntf.IXMLDocument;
begin
  LXMLDocument := Xml.XMLDoc.TXMLDocument.Create(nil);
  try
    LXMLDocument.LoadFromXML(AXML);
    LXMLDocument.Active := True;
    Result := FormatNode(LXMLDocument.DocumentElement);
  finally
    LXMLDocument := nil;
  end;
end;

function RemoveDTD(const AXML: string): string;
var
  StartPos, EndPos: Integer;
begin
  StartPos := Pos('<!DOCTYPE', UpperCase(AXML));

  if StartPos > 0 then
  begin
    EndPos := Pos('>', Copy(AXML, StartPos, Length(AXML) - StartPos + 1));
    if EndPos > 0 then
    begin
      Result := Copy(AXML, 1, StartPos - 1) +
                Copy(AXML, StartPos + EndPos, Length(AXML) - (StartPos + EndPos) + 1);
      Exit;
    end;
  end;

  Result := AXML;
end;

function FormatSVGXML(const AXML: string): string;
var
  LXMLDocument: Xml.XMLIntf.IXMLDocument;
  NoDTDXML: string;
begin
  NoDTDXML := RemoveDTD(AXML);

  LXMLDocument := TXMLDocument.Create(nil);
  try
    LXMLDocument.LoadFromXML(NoDTDXML);
    LXMLDocument.Active := True;
    LXMLDocument.Options := LXMLDocument.Options + [doNodeAutoIndent];
    Result := FormatSVG(LXMLDocument.XML.Text);
  finally
    LXMLDocument := nil;
  end;
end;

end.
